// src/utils/http.ts
import { useMemberStore } from '@/stores'
import { ENV } from '@/utils/env'
// 请求基地址  -- 基础路径+网口端
const baseURL = ENV.APP_TARGET + ENV.APP_PORT

// 拦截器配置
const httpInterceptor = {
  // 拦截前触发
  invoke(options: UniApp.RequestOptions) {
    // 1. 非 http 开头需拼接地址
    if (!options.url.startsWith('http')) {
      options.url = baseURL + options.url
    }
    // 2. 请求超时
    options.timeout = 10000
    // 3. 添加小程序端请求头标识
    options.header = {
      // 'source-client': 'miniapp',
      ...options.header,
    }
    // 4. 添加 token 请求头标识
    const memberStore = useMemberStore()
    const token = memberStore.profile
    if (token) {
      options.header.Authorization = 'Bearer ' + token // 让每个请求携带自定义token 请根据实际情况自行修改
    }
  },
}

// 拦截 request 请求
uni.addInterceptor('request', httpInterceptor)
// 拦截 uploadFile 文件上传
uni.addInterceptor('uploadFile', httpInterceptor)

/**
 * 请求函数
 * @param  UniApp.RequestOptions
 * @returns Promise
 *  1. 返回 Promise 对象，用于处理返回值类型
 *  2. 获取数据成功
 *    2.1 提取核心数据 res.data
 *    2.2 添加类型，支持泛型
 *  3. 获取数据失败
 *    3.1 401错误  -> 清理用户信息，跳转到登录页
 *    3.2 其他错误 -> 根据后端错误信息轻提示
 *    3.3 网络错误 -> 提示用户换网络
 */
type Data<T> = {
  code: string
  msg: string
  data: T
}
// 公共请求  2.2 添加类型，支持泛型
export const http = <T>(options: UniApp.RequestOptions) => {
  uni.showLoading({
    title: '加载中...',
    mask: true,
  })
  // 1. 返回 Promise 对象
  return new Promise<Data<T>>((resolve, reject) => {
    uni.request({
      ...options,
      // 响应成功
      success(res: any) {
        uni.hideLoading()
        console.log('500请求错误', res)
        // 状态码 2xx，参考 axios 的设计
        if (res.statusCode >= 200 && res.statusCode < 300) {
          // 2.1 提取核心数据 res.data
          // resolve(res.data as Data<T>)

          // ---------------响应是200 但数据内部是500|400----------------------------
          const resList = res.data

          if (resList.code === 200) {
            // uni.showToast({
            //   icon: 'none',
            //   title: '请求成功',
            // })
            resolve(res.data as Data<T>)
            return
          }

          if (resList.code === 401) {
            // 401错误  -> 清理用户信息，跳转到登录页
            uni.showToast({
              title: '请求错误',
              icon: 'error',
            })
            // const memberStore = useMemberStore()
            // memberStore.clearProfile()
            // setTimeout(() => {
            //   uni.navigateTo({ url: '/pages/login/OneClickLogin/OneClickLogin' })
            // }, 1000)
            reject(resList)
          } else {
            console.log('500请求错误', res.data)
            // 其他错误 -> 根据后端错误信息轻提示
            uni.showToast({
              icon: 'none',
              title: resList?.msg || '请求错误',
            })
            reject(resList)
          }

          // -------------------------------外部响应除200外---------------------------------------------------------------
        } else if (res.statusCode === 401) {
          // 401错误  -> 清理用户信息，跳转到登录页
          uni.showToast({
            title: (res.data as Data<T>).msg || '请求错误',
            icon: 'error',
          })
          // const memberStore = useMemberStore()
          // memberStore.clearProfile()
          // uni.navigateTo({ url: '/pages/login/OneClickLogin/OneClickLogin' })
          reject(res)
        } else {
          console.log('500请求错误', res.data)
          // 其他错误 -> 根据后端错误信息轻提示
          uni.showToast({
            icon: 'none',
            title: (res.data as Data<T>).msg || '请求错误',
          })
          reject(res)
        }
      },
      // 响应失败
      fail(err) {
        uni.showToast({
          icon: 'none',
          title: '网络错误，换个网络试试',
        })
        reject(err)
      },
    })
  })
}

// 上传图片接口
export const uploadRequest = <T>(options: any) => {
  // 1. 返回 Promise 对象
  return new Promise((resolve, reject) => {
    uni.uploadFile({
      url: baseURL + options.url, // 仅为示例，非真实的接口地址
      filePath: options.filePath,
      name: 'file',
      formData: {
        user: '',
      },
      success: (res) => {
        resolve(JSON.parse(res.data))
      },
      fail: (error) => {
        console.log('err' + error)
        reject(error)
      },
    })
  })
}
